;;; sr-org-kb-index.el

;; Generate a menufile for Marco Pratesi's great phplayersmenu.

;;; Usage
;;
;; The menu-structure file is made from the index-file of your project.
;; The index-filename is set in your `org-publish-project-alist':
;;
;; (require 'org-publish)
;;   (setq org-publish-project-alist
;;    '(("org-notes"
;;      :auto-index t ; generate index.org automagically
;;      :index-filename "sitemap.org"
;;      :recursive t
;;      :base-extension "org"
;;      :publishing-directory "~/develop/htdocs/org-notes/"
;;      :publishing-function org-publish-org-to-html
;;
;;      ;;
;;      ;; Layersmenu:
;;      ;;
;;      :completion-function sr-org-notes-kb-complete
;;      :layers-menu-structure-file "~/public_html/navigations/org.txt"
;;      ;; Optional:
;;      :layers-menu-link-target "mitte"
;;      )
;;
;; All other values used by phplayersmenu itself, may be set through stylesheets and
;; the phplayersmenu functions.
;;
;;;


(defun sr-org-notes-kb-complete ()
  "Take the index-file and turn it into the menu-structure
file for Marco Pratesi's phplayersmenu."
  (let* ((base-dir (file-name-as-directory (plist-get project-plist :base-directory)))
         (orig (expand-file-name (concat base-dir (plist-get project-plist :index-filename))))
         (strip-suffix (or (plist-get project-plist :base-extension) "org"))
         (add-suffix (or (plist-get project-plist :html-extension) "html"))

         (link-target (or (plist-get project-plist :layers-menu-link-target) "_blank"))
         (menu-file (or (plist-get project-plist :layers-menu-structure-file) nil))

         (visiting (find-buffer-visiting orig))
         (visiting-output (find-buffer-visiting menu-file))

         (input-buffer (find-file orig))
         (output-buffer (find-file menu-file))

         (old-ndots 1)
         (sub "")
         (old-sub ""))

    (unless menu-file
      (throw 'sr-org-note-kb-completion-error
             "No menu structure file provided. Giving up."))

    (with-current-buffer output-buffer
      (erase-buffer))

    (with-current-buffer input-buffer
      (widen)
      (goto-char (point-min))
      (while (re-search-forward org-bracket-link-analytic-regexp (point-max) t)
        (let ((link (match-string-no-properties 3))
              (text (match-string-no-properties 5))
              (pos 0)
              (ndots 1))

      (with-current-buffer output-buffer
        (if (string-match (concat "\\(" strip-suffix "\\)$") link)
            (setq link (replace-match add-suffix t t link)))
        (while (setq pos (string-match "/" link pos))
          (setq ndots (+ ndots 1))
          (setq pos (+ pos 1)))

        (when (< 1 ndots)
          (string-match "\\(/[^/]*\\)$" link)
          (setq sub (replace-match "" t t link))

          (unless (string= sub old-sub)
            (let ((ds 0)
                  (subs (split-string sub "/"))
                  (old-subs (split-string old-sub "/")))
              (while (string= (car old-subs) (car subs))
                (setq ds (+ ds 1))
                (pop old-subs)
                (pop subs))
              (dolist (d subs)
                (setq ds (+ ds 1))
                (insert
                 (concat
                  (make-string ds ?.) "|" d "\n")))
              (setq old-sub sub))))

        (insert
         (concat
          (make-string ndots ?.) "|" text "|" link "|||" link-target "\n"))
        (setq old-ndots ndots)
        ))))

    (or visiting (kill-buffer input-buffer))

    (with-current-buffer output-buffer
      (save-buffer))
    (or visiting-output (kill-buffer output-buffer))
))


(provide 'sr-org-index-complete)
